"use client";
import React from "react";
import {Label} from "@/components/ui/label";
import {Input} from "@/components/ui/input";
import {cn} from "@/lib/utils";
import {BadgeAlert, OctagonX, User} from "lucide-react";
import Link from "next/link";
import {Button} from "@/components/ui/button";
import {SubmitHandler, useForm} from "react-hook-form";
import toast, {ToastBar} from "react-hot-toast";
import {ToastBox, toastType} from "@/components/ui/toast-box";
import {zodResolver} from "@hookform/resolvers/zod"
import {z} from 'zod'
import {AccountService} from "@/services/account.service";

const Schema = z.object({
	username:
		z.string()
		.min(3, {
			message: 'The username should be at least 3 character(s).'
		})
		.max(14, {
			message: 'The username should be at most 14 character(s).'
		}),
  displayName:
		z.string()
		.min(3, {
			message: 'The display name should be at least 3 character(s).'
		})
		.max(20, {
			message: 'The display name should be at most 20 character(s).'
		})
		.optional(),
  email:
		z.string()
		.email({
			message: 'Please enter a valid email address.'
		})
		.max(32, {
			message: 'The email should be less than 32 character(s).'
		}),
  password:
		z.string()
		.min(8, {
			message: 'The password should be at least 8 character(s).'
		})
		.max(32, {
			message: 'The password should be less than 32 character(s).'
		}),
  rePassword:
		z.string()
}).superRefine(({ rePassword, password }, ctx) => {
	if (rePassword !== password) {
		ctx.addIssue({
			code: "custom",
			message: "The passwords did not match",
			path: ['rePassword']
		});
	}});

type UserInsert = z.infer<typeof Schema>

function ErrorLabel({message}: {message: string}) {
	return (
		<div className={'flex flex-row items-center gap-1 text-sm'}>
			<BadgeAlert className={'text-red-700'}/>
			<p className={'text-red-700'}>{message}</p>
		</div>
	)
}

export function RegisterForm() {

	const {
		register,
		handleSubmit,
		getValues,
		setError,
		formState: {errors,
			isSubmitting} } = useForm<UserInsert>({
		resolver: zodResolver(Schema)
	});

	const onSubmit: SubmitHandler<UserInsert> = async (data) => {
			console.log('ping');
			const result = await AccountService.register(data)
			console.log(result);
			if (!result.success) {
				toast.custom(<ToastBox
					message={result.message}
					type={toastType.refused}
				/>)
				return
			}
			toast.custom(<ToastBox
				title={`Verify your email`}
				message={`Please verify you mail inbox to activate your account.`}
				type={toastType.success}
			/>)
			console.log(data);
			//setError('root', {message: 'An error occurred.'})
	}
	return (
		<div className="max-w-md w-full mx-auto rounded-none md:rounded-2xl p-4 md:p-8 shadow-input md:scale-125">
			<h2 className="font-bold text-xl text-neutral-800 dark:text-neutral-200">
				Create an account
			</h2>
			<p className="text-neutral-600 text-sm max-w-sm mt-2 dark:text-neutral-300">
				By creating an account you can discuss on our platform.
			</p>

			<form className="my-8" onSubmit={handleSubmit(onSubmit)}>
				<div className="flex flex-col md:flex-row space-y-2 md:space-y-0 md:space-x-2 mb-1">
					<LabelInputContainer className="mb-4">
						<Label htmlFor="username">Username<em className={"opacity-50"}>*</em></Label>
						<Input
							id="username"
							placeholder="john.doe68"
							type="text"
							{...register("username")}
						/>
					</LabelInputContainer>
					<LabelInputContainer className="mb-4">
						<Label htmlFor="displayName">Displayname</Label>
						<Input
							id="displayName"
							placeholder="Johny"
							type="text"
							{...register("displayName", {required: false})}
						/>
					</LabelInputContainer>
				</div>
				{errors.username && <ErrorLabel message={errors.username?.message || ''} />}
				<div
					className="bg-gradient-to-r from-transparent via-neutral-300 dark:via-neutral-700 to-transparent my-4 h-[1px] w-full"/>
				<LabelInputContainer className="mb-4">
					<Label htmlFor="email">Email Address<em className={"opacity-50"}>*</em></Label>
					<Input
						id="email"
						placeholder="john.doe@foo.bar"
						type="email"
						{...register("email")}
					/>
					{errors.email && <ErrorLabel message={errors.email?.message || ''} />}
				</LabelInputContainer>
				<LabelInputContainer className="mb-4">
					<Label htmlFor="password">Password<em className={"opacity-50"}>*</em></Label>
					<Input
						id="password"
						placeholder="••••••••"
						type="password"
						{...register("password")}
					/>
					{errors.password && <ErrorLabel message={errors.password?.message || ''} />}
				</LabelInputContainer>
				<LabelInputContainer className="mb-8">
					<Label htmlFor="re-password">Re-type password<em className={"opacity-50"}>*</em></Label>
					<Input
						id="rePassword"
						placeholder="••••••••"
						type="password"
						{...register("rePassword", {required: 'You need fill again our password here.', validate: (value) => value === getValues('password') || "The passwords do not match." })}
					/>
					{errors.rePassword && <ErrorLabel message={errors.rePassword?.message || ''} />}
				</LabelInputContainer>
				<div className={'flex flex-row justify-center items-center'}>
					{errors.root && <ErrorLabel message={errors.root?.message || ''} />}
				</div>
				<button
					className="bg-gradient-to-br relative group/btn from-black dark:from-zinc-900 dark:to-zinc-900 to-neutral-600 block dark:bg-zinc-800 w-full text-white rounded-md h-10 font-medium shadow-[0px_1px_0px_0px_#ffffff40_inset,0px_-1px_0px_0px_#ffffff40_inset] dark:shadow-[0px_1px_0px_0px_var(--zinc-800)_inset,0px_-1px_0px_0px_var(--zinc-800)_inset]"
					type="submit"
				>
					{!isSubmitting ? 'Register' : 'Loading...'}
					<BottomGradient/>
				</button>
				<div
					className="bg-gradient-to-r from-transparent via-neutral-300 dark:via-neutral-700 to-transparent my-8 h-[1px] w-full"/>

				<div className="flex flex-col space-y-4">
					<Button
						className=" relative group/btn flex space-x-2 items-center justify-center px-4 w-full text-black rounded-md h-10 font-medium shadow-input bg-gray-50 dark:bg-zinc-900 dark:shadow-[0px_0px_1px_1px_var(--neutral-800)]"
						asChild
					>
						<Link
							className={'flex flex-row justify-center items-center gap-2'}
							href={"/account/login"}>
							<User className="h-4 w-4 text-neutral-800 dark:text-neutral-300"/>
							<span className="text-neutral-700 dark:text-neutral-300 text-sm">
              I already have an account
            	</span>
							<BottomGradient/>
						</Link>
					</Button>
				</div>
			</form>
		</div>
	);
}

const BottomGradient = () => {
	return (
		<>
			<span className="group-hover/btn:opacity-100 block transition duration-500 opacity-0 absolute h-px w-full -bottom-px inset-x-0 bg-gradient-to-r from-transparent via-accent-foreground to-transparent" />
			<span className="group-hover/btn:opacity-100 blur-sm block transition duration-500 opacity-0 absolute h-px w-1/2 mx-auto -bottom-px inset-x-10 bg-gradient-to-r from-transparent via-primary-foreground to-transparent" />
		</>
	);
};

const LabelInputContainer = ({
															 children,
															 className,
														 }: {
	children: React.ReactNode;
	className?: string;
}) => {
	return (
		<div className={cn("flex flex-col space-y-2 w-full", className)}>
			{children}
		</div>
	);
};
